from PropBank.Role import Role


class RoleSet(object):

    __id: str
    __name: str
    __roles: list

    """
    A constructor of RoleSet class which takes id and name as inputs and initializes corresponding attributes
    with these inputs.

    PARAMETERS
    ----------
    id : str
        Id of the roleSet
    name : str
        Name of the roleSet
    """
    def __init__(self, _id: str, name: str):
        self.__id = _id
        self.__name = name
        self.__roles = []

    """
    Accessor for id.

    RETURNS
    -------
    str
        id.
    """
    def getId(self) -> str:
        return self.__id

    """
    Accessor for name.

    RETURNS
    -------
    str
        name.
    """
    def getName(self) -> str:
        return self.__name

    """
    The size method returns the size of the roles list.

    RETURNS
    -------
    int
        the size of the roles list.
    """
    def size(self) -> int:
        return len(self.__roles)

    """
    The addRole method takes a Role as input and adds it to the roles list.

    PARAMETERS
    ----------
    role : Role 
        Role to be added
    """
    def addRole(self, role: Role):
        self.__roles.append(role)

    """
    The getRole method returns the role at the given index.

    PARAMETERS
    ----------
    index : int 
        Index of the role
        
    RETURNS
    -------
    Role
        Role at the given index.
    """
    def getRole(self, index: int) -> Role:
        return self.__roles[index]

    """
    Finds and returns the role with the given argument number n. For example, if n == 0, the method returns
    the argument ARG0.
    
    PARAMETERS
    ----------
    n : str
        Argument number
        
    RETURNS
    -------
    Role
        The role with the given argument number n.
    """
    def getRoleWithArgument(self, n: str) -> Role:
        for role in self.__roles:
            if role.getN() == n:
                return role



from __future__ import annotations
from enum import Enum, auto

"""
Enumerated class for argument type.
"""


class ArgumentType(Enum):
    NONE = auto()
    PREDICATE = auto()
    ARG0 = auto()
    ARG1 = auto()
    ARG2 = auto()
    ARG3 = auto()
    ARG4 = auto()
    ARG5 = auto()
    ARGMNONE = auto()
    ARGMEXT = auto()
    ARGMLOC = auto()
    ARGMDIS = auto()
    ARGMADV = auto()
    ARGMCAU = auto()
    ARGMTMP = auto()
    ARGMPNC = auto()
    ARGMMNR = auto()
    ARGMDIR = auto()

    """
    The getArguments method takes an argumentsType string and returns the ArgumentType form of it.

    PARAMETERS
    ----------
    argumentsType : str 
        Type of the argument in string form
        
    RETURNS
    -------
    ArgumentType
        Type of the argument in ArgumentType form
    """
    @staticmethod
    def getArguments(argumentsType: str) -> ArgumentType:
        for argumentType in ArgumentType:
            if argumentsType == argumentType.name:
                return argumentType
        return ArgumentType.NONE

    """
    The getPropbankType method takes an argumentType in ArgumentType form and returns the string form of it.

    PARAMETERS
    ----------
    argumentType : ArgumentType 
        Type of the argument in {@link ArgumentType} form
        
    RETURNS
    -------
    str
        Type of the argument in string form
    """
    @staticmethod
    def getPropbankType(argumentType: ArgumentType) -> str:
        if argumentType is None:
            return "NONE"
        return argumentType.name



from PropBank.Frameset import Frameset
import os


class FramesetList(object):

    __frames: list

    """
    A constructor of FramesetList class which reads all frameset files inside the Predicates folder. For each
    file inside that folder, the constructor creates a Frameset and puts in inside the frames list.
    """
    def __init__(self):
        self.__frames = []
        for r, d, f in os.walk("Predicates/"):
            for file in f:
                frameset = Frameset("")
                frameset.initWithFile(os.path.join(r, file))
                self.__frames.append(frameset)

    """
    readFromXmL method searches the Frameset with a given synSetId if there is a Frameset with the given synSet id,
    returns the arguments of that Frameset as a dictionary.

    PARAMETERS
    ----------
    synSetId : str 
        Id of the searched Frameset
        
    RETURNS
    -------
    dict
        a dict containing the arguments of the searched Frameset
    """
    def readFromXml(self, synSetId: str) -> dict:
        frameset = {}
        for f in self.__frames:
            if f.getId() == synSetId:
                for i in range(len(f.getFramesetArguments())):
                    framesetArgument = f.getFramesetArguments()[i]
                    frameset[framesetArgument.getArgumentType()] = framesetArgument.getDefinition()
        return frameset

    """
    frameExists method checks if there is a Frameset with the given synSet id.

    PARAMETERS
    ----------
    synSetId : str 
        Id of the searched Frameset
        
    RETURNS
    -------
    bool
        true if the Frameset with the given id exists, false otherwise.
    """
    def frameExists(self, synSetId: str) -> bool:
        for f in self.__frames:
            if f.getId == synSetId:
                return True
        return False

    """
    getFrameSet method returns the Frameset with the given synSet id.

    PARAMETERS
    ----------
    synSetId : str 
        Id of the searched Frameset
        
    RETURNS
    -------
    Frameset
        Frameset which has the given id.
    """
    def getFrameSet(self, synSetId: str) -> Frameset:
        for f in self.__frames:
            if f.getId == synSetId:
                return f
        return None

    """
    The addFrameset method takes a Frameset as input and adds it to the frames list.

    PARAMETERS
    ----------
    frameset : Frameset 
        Frameset to be added
    """
    def addFrameset(self, frameset: Frameset):
        self.__frames.append(frameset)

    """
    The getFrameSet method returns the frameset at the given index.

    PARAMETERS
    ----------
    index : int 
        Index of the frameset

    RETURNS
    -------
    Frameset
        Frameset at the given index.
    """
    def getFrameset(self, index: int) -> Frameset:
        return self.__frames[index]

    """
    The size method returns the size of the frames list.

    RETURNS
    -------
    int
        the size of the frames list.
    """
    def size(self) -> int:
        return len(self.__frames)



class FramesetArgument(object):

    __argumentType: str
    __definition: str

    """
    A constructor of FramesetArgument class which takes argumentType and definition as input and initializes
    corresponding attributes

    PARAMETERS
    ----------
    argumentType : str
        ArgumentType of the frameset argument
    definition : str
        Definition of the frameset argument
    """
    def __init__(self, argumentType: str, definition: str):
        self.__argumentType = argumentType
        self.__definition = definition

    """
    Accessor for argumentType.

    RETURNS
    -------
    str
        argumentType.
    """
    def getArgumentType(self) -> str:
        return self.__argumentType

    """
    Accessor for definition.

    RETURNS
    -------
    str
        definition.
    """
    def getDefinition(self) -> str:
        return self.__definition

    """
    Mutator for definition.
    
    PARAMETERS
    ----------
    definition : str
        definition to set.
    """
    def setDefinition(self, definition: str):
        self.__definition = definition

    """
    __str__ converts an FramesetArgument to a string. It returns argument string which is in the form of
    argumentType:definition

    RETURNS
    -------
    str
        string form of frameset argument
    """
    def __str__(self) -> str:
        return self.__argumentType + ":" + self.__definition



class Argument(object):

    __argumentType: str
    __id: str

    """
    A constructor of Argument class which takes argument string which is in the form of argumentType$id
    and parses this string into argumentType and id. If the argument string does not contain '$' then the
    constructor return a NONE type argument.

    PARAMETERS
    ----------
    argument : str
        Argument string containing the argumentType and id
    """
    def __init__(self, argument: str):
        if "$" in argument:
            self.__argumentType = argument[0:argument.index("$")]
            self.__id = argument[argument.index("$") + 1:]
        else:
            self.__argumentType = "NONE"

    """
    Another constructor of Argument class which takes argumentType and id as inputs and initializes corresponding attributes

    PARAMETERS
    ----------
    argumentType : str 
        Type of the argument
    id : str 
        Id of the argument
    """
    def initWithId(self, argumentType: str, id: str):
        self.__argumentType = argumentType
        self.__id = id

    """
    Accessor for argumentType.
    
    RETURNS
    -------
    str
        argumentType.
    """
    def getArgumentType(self) -> str:
        return self.__argumentType

    """
    Accessor for id.

    RETURNS
    -------
    str
        id.
    """
    def getId(self) -> str:
        return self.__id

    """
    __str__ converts an Argument to a string. If the argumentType is "NONE" returns only "NONE", otherwise
    it returns argument string which is in the form of argumentType$id

    RETURNS
    -------
    str
        string form of argument
    """
    def __str__(self) -> str:
        if self.__argumentType == "NONE":
            return self.__argumentType
        else:
            return self.__argumentType + "$" + self.__id


from PropBank.RoleSet import RoleSet


class Predicate(object):

    __lemma: str
    __roleSets: list

    """
    A constructor of Predicate class which takes lemma as input and initializes lemma with this input.
    The constructor also initializes the roleSets array.

    PARAMETERS
    ----------
    lemma : str
        Lemma of the predicate
    """
    def __init__(self, lemma: str):
        self.__lemma = lemma
        self.__roleSets = []

    """
    Accessor for lemma.

    RETURNS
    -------
    str
        lemma.
    """
    def getLemma(self) -> str:
        return self.__lemma

    """
    The addRoleSet method takes a RoleSet as input and adds it to the roleSets list.

    PARAMETERS
    ----------
    roleSet : RoleSet 
        RoleSet to be added
    """
    def addRoleSet(self, roleSet: RoleSet):
        self.__roleSets.append(roleSet)

    """
    The size method returns the size of the roleSets list.

    RETURNS
    -------
    int
        the size of the roleSets list.
    """
    def size(self) -> int:
        return len(self.__roleSets)

    """
    The getRoleSet method returns the roleSet at the given index.

    PARAMETERS
    ----------
    index : int 
        Index of the roleSet
        
    RETURNS
    -------
    RoleSet
        RoleSet at the given index.
    """
    def getRoleSet(self, index: int) -> RoleSet:
        return self.__roleSets[index]

    """
    Another getRoleSet method which returns the roleSet with the given roleSet id.

    PARAMETERS
    ----------
    roleId : str 
        Id of the searched roleSet

    RETURNS
    -------
    RoleSet
        RoleSet which has the given id.
    """
    def getRoleSetWithId(self, roleId: str) -> RoleSet:
        for roleSet in self.__roleSets:
            if roleSet.getId() == roleId:
                return roleSet
        return None



from PropBank.ArgumentType import ArgumentType


class Role(object):

    __description: str
    __f: str
    __n: str

    """
    A constructor of Role class which takes description, f, and n as inputs and initializes corresponding with
    these inputs.

    PARAMETERS
    ----------
    description : str
        Description of the role
    f : str
        Argument Type of the role
    n : str
        Number of the role
    """
    def __init__(self, description: str, f: str, n: str):
        self.__description = description
        self.__f = f
        self.__n = n

    """
    Accessor for description.

    RETURNS
    -------
    str
        description.
    """
    def getDescription(self) -> str:
        return self.__description

    """
    Accessor for f.

    RETURNS
    -------
    str
        f.
    """
    def getF(self) -> str:
        return self.__f

    """
    Accessor for n.

    RETURNS
    -------
    str
        n.
    """
    def getN(self) -> str:
        return self.__n

    """
    Constructs and returns the argument type for this role.

    RETURNS
    -------
    ArgumentType
        Argument type for this role.
    """
    def getArgumentType(self) -> ArgumentType:
        return ArgumentType.getArguments("ARG" + self.__f.upper())



from PropBank.ArgumentType import ArgumentType
from PropBank.FramesetArgument import FramesetArgument
import xml.etree.ElementTree


class Frameset(object):

    __framesetArguments: list
    __id: str

    """
    A constructor of Frameset class which takes id as input and initializes corresponding attribute

    PARAMETERS
    ----------
    id : str
        Id of the frameset
    """
    def __init__(self, _id: str):
        self.__id = _id
        self.__framesetArguments = []

    """
    Another constructor of Frameset class which takes filename as input and reads the frameset

    PARAMETERS
    ----------
    fileName : str  
        File name of the file to read frameset
    """
    def initWithFile(self, fileName: str):
        root = xml.etree.ElementTree.parse(fileName).getroot()
        self.__id = root.attrib["id"]
        for child in root:
            self.__framesetArguments.append(FramesetArgument(child.attrib["name"], child.text))

    """
    containsArgument method which checks if there is an Argument of the given argumentType.

    PARAMETERS
    ----------
    argumentType : ArgumentType 
        ArgumentType of the searched Argument
        
    RETURNS
    -------
    bool
        true if the Argument with the given argumentType exists, false otherwise.
    """
    def containsArgument(self, argumentType: ArgumentType) -> bool:
        for framesetArgument in self.__framesetArguments:
            if ArgumentType.getArguments(framesetArgument.getArgumentType()) == argumentType:
                return True
        return False

    """
    The addArgument method takes a type and a definition of a FramesetArgument as input, then it creates a new 
    FramesetArgument from these inputs and adds it to the framesetArguments list.

    PARAMETERS
    ----------
    type : str 
        Type of the new FramesetArgument
    definition : str
        Definition of the new FramesetArgument
    """
    def addArgument(self, argumentType: str, definition: str):
        check = False
        for framesetArgument in self.__framesetArguments:
            if framesetArgument.getArgumentType() == argumentType:
                framesetArgument.setDefinition(definition)
                check = True
                break
        if not check:
            arg = FramesetArgument(argumentType, definition)
            self.__framesetArguments.append(arg)

    """
    The deleteArgument method takes a type and a definition of a FramesetArgument as input, then it searches for the 
    FramesetArgument with these type and definition, and if it finds removes it from the framesetArguments list.

    PARAMETERS
    ----------
    type : str 
        Type of the to be deleted FramesetArgument
    definition : str 
        Definition of the to be deleted FramesetArgument
    """
    def deleteArgument(self, argumentType: str, definition: str):
        for framesetArgument in self.__framesetArguments:
            if framesetArgument.getArgumentType() == argumentType and framesetArgument.getDefinition() == definition:
                self.__framesetArguments.remove(framesetArgument)
                break

    """
    Accessor for framesetArguments.

    RETURNS
    -------
    list
        framesetArguments.
    """
    def getFramesetArguments(self) -> list:
        return self.__framesetArguments

    """
    Accessor for id.

    RETURNS
    -------
    str
        id.
    """
    def getId(self) -> str:
        return self.__id

    """
    Mutator for id.

    PARAMETERS
    ----------
    id : str 
        id to set.
    """
    def setId(self, _id: str):
        self.__id = _id
